Bartender3 = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0", "AceDebug-2.0", "AceModuleCore-2.0")
Bartender3:SetModuleMixins("AceEvent-2.0")
--[[
	Layouts
		0 = simple
		1 = enhanced
		2 = advanced
]]

function Bartender3:OnInitialize()
	self:SetDebugging(true)
	self:RegisterDB("BT3DB")
	
	_, self.playerclass = UnitClass("player")
	
	self:SetupDefaults()
	
	self.unlock = false
	self.actionbars, self.actionbuttons, self.specialbars, self.bars = {}, {}, {}, {}
	for i=1,120 do
		self.actionbuttons[i] = { state = "unused", object = nil}
	end
	if self.db.profile.Layout == 0 then
		for i=1,10 do
			if self.db.profile.Bars[i].Enabled then
				local bar = Bartender3.Class.ActionBar:new(i, self.db.profile.Bars[i], self.db.profile.Bars[i].Buttons or 12, (i == 1) and true or false)
				self.actionbars[i] = bar
			end
		end
	else
		-- TODO Advanced
	end
	
	self:RemoveDefaultUIElements()
	
	self:PopulateOptions()
end

function Bartender3:RemoveDefaultUIElements()
	-- Hide the main grafics
	MainMenuBarArtFrame:Hide()
	MainMenuBar:Hide()
	
	self.specialbars["STANCE"] = 	Bartender3.Class.SpecialBar:new("STANCE", "StanceBar", self.db.profile.SpecialBars["STANCE"])
	self.specialbars["PET"] = 		Bartender3.Class.SpecialBar:new("PET", "PetBar", self.db.profile.SpecialBars["PET"])
	self.specialbars["MICROMENU"] =	Bartender3.Class.SpecialBar:new("MICROMENU", "MicroMenu", self.db.profile.SpecialBars["MICROMENU"])
	self.specialbars["BAGS"] =		Bartender3.Class.SpecialBar:new("BAGS", "BagBar", self.db.profile.SpecialBars["BAGS"])
end

function Bartender3:OnEnable()
	self:RefreshBars()
end

function Bartender3:OnProfileEnable()
	for i,v in pairs(self.actionbars) do
		v:UpdateProfile(self.db.profile.Bars[i])
	end
	for i,v in pairs(self.specialbars) do
		v:UpdateProfile(self.db.profile.specialbars[i])
	end
end

function Bartender3:GetFreeButton(bar)
	if self.db.profile.Layout == 0 and bar then
		for i=1,12 do
			local button = ((bar.id-1)*12)+i
			if ( self.actionbuttons[button].state == "unused" ) then 
				local newbutton = Bartender3.Class.Button:new(bar, button)
				self.actionbuttons[button].state = "used"
				self.actionbuttons[button].object = newbutton
				return newbutton
			elseif ( self.actionbuttons[button].state == "free" ) then 
				self.actionbuttons[button].state = "used"
				-- do parent setting and stuff here - TODO - not used in any function yet
				return self.actionbuttons[button].object
			end
		end
	end
end

function Bartender3:Unlock()
	for i,v in pairs(self.actionbars) do
		if v then v:UnlockFrames() end
	end
	for i,v in pairs(self.specialbars) do
		if v then v:UnlockFrames() end
	end
	self.unlock = true
end

function Bartender3:Lock()
	for i,v in pairs(self.actionbars) do
		if v then v:LockFrames() end
	end
	for i,v in pairs(self.specialbars) do
		if v then v:LockFrames() end
	end
	self.unlock = nil
end

function Bartender3:RefreshBars(bar)
	if bar then bar:RefreshLayout() else
		for i,v in pairs(self.actionbars) do
			v:RefreshLayout()
		end
		for i,v in pairs(self.specialbars) do
			v:RefreshLayout()
		end
	end
end

function Bartender3:GetConfig(index, configtype)
	if configtype == 0 then
		return self.db.profile.Bars[index]
	elseif configtype == 1 then
		return self.db.profile.SpecialBars[index]
	end
end

function Bartender3:SetZoom(zoom)
	self.db.profile.Zoomed = zoom
	for i,v in pairs(self.actionbars) do
		if v then v:ButtonZoomCheck() end
	end
	for i,v in pairs(self.specialbars) do
		if v then v:ButtonZoomCheck() end
	end
end

function Bartender3:SwitchSelfCastModifier(scm)
	local oldscm = self.db.profile.SelfCastModifier
	self.db.profile.SelfCastModifier = scm
	for i,v in pairs(self.actionbars) do
		if v then 
			v.frame:SetAttribute(oldscm.."-unit*", nil) 
			v:UpdateSelfCast()
		end
	end
end
